'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_NV_shader_thread_group'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_NV_shader_thread_group',error_checker=_errors._error_checker)
GL_SM_COUNT_NV=_C('GL_SM_COUNT_NV',0x933B)
GL_WARPS_PER_SM_NV=_C('GL_WARPS_PER_SM_NV',0x933A)
GL_WARP_SIZE_NV=_C('GL_WARP_SIZE_NV',0x9339)

